package com.persagy.dmp.alarm.service;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.common.criteria.CriteriaUtils;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.date.DateUtils;
import com.persagy.common.web.MapResponse;
import com.persagy.common.web.PagedResponse;
import com.persagy.dmp.alarm.entity.AlarmRecord;
import com.persagy.dmp.alarm.entity.QAlarmRecord;
import com.persagy.dmp.alarm.enumeration.EnumAlarmLogType;
import com.persagy.dmp.alarm.enumeration.EnumAlarmMessageType;
import com.persagy.dmp.alarm.enumeration.EnumAlarmRecordWithColumns;
import com.persagy.dmp.alarm.model.AlarmLogModel;
import com.persagy.dmp.alarm.model.AlarmRecordModel;
import com.persagy.dmp.alarm.repository.AlarmRecordRepository;
import com.persagy.dmp.alarm.util.CheckRequiredParam;
import com.persagy.dmp.config.DmpParameterStorage;
import com.persagy.dmp.rwd.model.DmpMessage;
import com.querydsl.core.types.dsl.BooleanExpression;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.transaction.Transactional;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class AlarmRecordService extends AlarmBaseService {

	@Autowired
	private AlarmRecordRepository alarmRecordRepository;
	@Autowired
	private AlarmLogService alarmLogService;
	@Autowired
	private CriteriaUtils criteriaUtils;

	private List<BooleanExpression> parse(ObjectNode criteria) {
		// TODO
		return null;
	}

	public PagedResponse<AlarmRecordModel> query(JacksonCriteria criteria) {
		PagedResponse<AlarmRecord> resp = criteriaUtils.query(QAlarmRecord.alarmRecord, this::parse, criteria);
		PagedResponse<AlarmRecordModel> result = new PagedResponse<>();
		result.setCount(resp.getCount());
		List<AlarmRecord> dataList = resp.getData();
		// 如果需要查询日志相关的信息，需使用withColumns参数
		Set<String> withColumns = criteria.getWithColumns();
		if (dataList != null && dataList.size() > 0) {
			List<AlarmRecordModel> collect = dataList.stream().map(entity -> {
				AlarmRecordModel model = entity.toModel();
				if (withColumns.contains(EnumAlarmRecordWithColumns.ignoreTime.name())) {
					Date ignoreAlarmTime = alarmLogService.getIgnoreAlarmTime(model.getId());
					model.setIgnoreTime(ignoreAlarmTime);
				}
				if (withColumns.contains(EnumAlarmRecordWithColumns.treatEndTime.name())) {
					Date treatEndTime = alarmLogService.getTreatEndTime(model.getId());
					model.setTreatEndTime(treatEndTime);
				}
				return model;
			}).collect(Collectors.toList());
			result.setData(collect);
		}
		return result;
	}

    @Transactional
    public MapResponse create(AlarmRecordModel param) {
        MapResponse response = new MapResponse();
        AlarmParam alarmParam = prepareParam(response);
        if (alarmParam == null) {
            return response;
        }
        // 必填项校验
		String checkResult = CheckRequiredParam.check(param, "itemCode", "objId", "level");;
        if (!StringUtils.isEmpty(checkResult)) {
        	response.setFail(checkResult);
        	return response;
		}

        String id = UUID.randomUUID().toString();
        // 如果创建时，入参包含id,则使用入参中的id。但需要先校验id是否已存在
        if (!StringUtils.isEmpty(param.getId())) {
			Optional<AlarmRecord> byIdResult = alarmRecordRepository.findById(param.getId());
			if (byIdResult.isPresent()) {
				response.setFail("已存在相同id的报警记录！创建失败！");
			} else {
				id = param.getId();
			}
		}
		AlarmRecord entity = AlarmRecord.fromModel(param);
        entity.setId(id);
        entity.setCreateUser(alarmParam.userId);
        entity.setCreateTime(new Date());
        entity.setValid(1);
        entity.setProjectId(alarmParam.projectId);
        entity.setGroupCode(alarmParam.groupCode);
        alarmRecordRepository.save(entity);
        response.add("id", entity.getId());
		// 推送消息至mq
		DmpMessage msg = new DmpMessage();
		msg.setMid(UUID.randomUUID().toString());
		msg.setType(EnumAlarmMessageType.ALARM_RECORD_CREATE.getValue());
		msg.setGroupCode(DmpParameterStorage.getGroupCode());
		msg.setProjectId(DmpParameterStorage.getProjectId());
		msg.setTargetId(entity.getId());
		msg.add("obj", JSONObject.toJSONString(entity));
		msg.setAppId(DmpParameterStorage.getAppId());
		msg.setSendTime(DateUtils.format(new Date()));
		response.add(msg);
        return response;
    }

	public AlarmRecord get(String id) {
		Optional<AlarmRecord> result = alarmRecordRepository.findById(id);
		return result.orElse(null);
	}


	/**
	 * @description: 初始化日志对象
	 * @param: param 报警记录参数
	 * @return: com.persagy.dmp.alarm.model.AlarmLogModel
	 * @exception:
	 * @author: lixing
	 * @company: Persagy Technology Co.,Ltd
	 * @since: 2020/12/8 3:07 下午
	 * @version: V1.0
	 */
	private AlarmLogModel initAlarmLogModel(AlarmRecordModel param) {
		AlarmLogModel alarmLogModel = new AlarmLogModel();
		alarmLogModel.setRecordId(param.getId());
		alarmLogModel.setCreateTime(new Date());
		alarmLogModel.setCreateUser(DmpParameterStorage.getUserId());
		alarmLogModel.setProjectId(DmpParameterStorage.getProjectId());
		alarmLogModel.setGroupCode(DmpParameterStorage.getGroupCode());
		return alarmLogModel;
	}

	@Transactional
	public MapResponse update(AlarmRecordModel param) {
		MapResponse response = new MapResponse();
		AlarmRecord alarmRecord = get(param.getId());
		if (alarmRecord == null) {
			response.setFail("无法获取到要更新的数据！");
			return response;
		}
		AlarmParam alarmParam = prepareParam(response);
		if (alarmParam == null) {
			return response;
		}
		if (param.getName() != null && !param.getName().isEmpty()) {
			alarmRecord.setName(param.getName());
		}
		if (param.getRemark() != null && !param.getRemark().isEmpty()) {
			alarmRecord.setRemark(param.getRemark());
		}
		// 状态修改记录日志
		if (param.getState() != null) {
			if (!param.getState().equals(alarmRecord.getState())) {
				AlarmLogModel alarmLogModel = initAlarmLogModel(param);
				alarmLogModel.setType(EnumAlarmLogType.ALARMSTATE);
				alarmLogModel.setNature(alarmRecord.getNature());
				alarmLogModel.setBefore(alarmRecord.getState());
				alarmLogModel.setAfter(param.getState());
				alarmLogService.create(alarmLogModel);
			}
			alarmRecord.setState(param.getState());
		}
		if (param.getNature() != null) {
			alarmRecord.setNature(param.getNature());
		}
		// 处理方式修改记录日志
		if (param.getTreatMode() != null) {
			if (!param.getTreatMode().equals(alarmRecord.getTreatMode())) {
				AlarmLogModel alarmLogModel = initAlarmLogModel(param);
				alarmLogModel.setType(EnumAlarmLogType.TREATMODE);
				alarmLogModel.setNature(alarmRecord.getNature());
				alarmLogModel.setBefore(alarmRecord.getTreatMode());
				alarmLogModel.setAfter(param.getTreatMode());
				alarmLogService.create(alarmLogModel);
			}
			alarmRecord.setTreatMode(param.getTreatMode());
		}
		// 处理状态更改记录日志
		if (param.getTreatState() != null) {
			if (!param.getTreatState().equals(alarmRecord.getTreatState())) {
				AlarmLogModel alarmLogModel = initAlarmLogModel(param);
				alarmLogModel.setType(EnumAlarmLogType.TREATSTATE);
				alarmLogModel.setNature(alarmRecord.getNature());
				alarmLogModel.setBefore(alarmRecord.getTreatState());
				alarmLogModel.setAfter(param.getTreatState());
				alarmLogService.create(alarmLogModel);
			}
			alarmRecord.setTreatState(param.getTreatState());
		}

		// 恢复值
		if (param.getEndInfo() != null) {
			alarmRecord.setEndInfo(param.getEndInfo());
		}
		// 恢复时间
		if (param.getEndTime() != null) {
			alarmRecord.setEndTime(param.getEndTime());
		}
		// 工单id
		if (param.getOrderId() != null) {
			alarmRecord.setOrderId(param.getOrderId());
		}
		// 工单状态
		if (param.getOrderState() != null) {
			alarmRecord.setOrderState(param.getOrderState());
		}
		// 补充说明
		if (param.getSupplement() != null) {
			alarmRecord.setSupplement(param.getSupplement());
		}
		alarmRecord.setUpdateUser(alarmParam.userId);
		alarmRecord.setUpdateTime(new Date());
		alarmRecordRepository.save(alarmRecord);
		response.add("id", alarmRecord.getId());
		// 推送消息至mq
		DmpMessage msg = new DmpMessage();
		msg.setMid(UUID.randomUUID().toString());
		msg.setType(EnumAlarmMessageType.ALARM_RECORD_UPDATE.getValue());
		msg.setGroupCode(DmpParameterStorage.getGroupCode());
		msg.setProjectId(DmpParameterStorage.getProjectId());
		msg.setTargetId(alarmRecord.getId());
		msg.add("obj", JSONObject.toJSONString(alarmRecord));
		msg.setAppId(DmpParameterStorage.getAppId());
		msg.setSendTime(DateUtils.format(new Date()));
		response.add(msg);
		return response;
	}

	//	@Transactional
	//	public MapResponse delete(AlarmRecordModel param) {
	//		MapResponse response = new MapResponse();
	//		// TODO
	//		alarmRecordRepository.deleteById(param.getId());
	//		return response;
	//	}
}

