package com.persagy.dmp.alarm.service;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.common.criteria.CriteriaUtils;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.date.DateUtils;
import com.persagy.common.web.MapResponse;
import com.persagy.common.web.PagedResponse;
import com.persagy.dmp.alarm.entity.AlarmTarget;
import com.persagy.dmp.alarm.entity.QAlarmTarget;
import com.persagy.dmp.alarm.enumeration.EnumAlarmMessageType;
import com.persagy.dmp.alarm.model.AlarmItemModel;
import com.persagy.dmp.alarm.model.AlarmTargetModel;
import com.persagy.dmp.alarm.repository.AlarmTargetRepository;
import com.persagy.dmp.config.DmpParameterStorage;
import com.persagy.dmp.rwd.model.DmpMessage;
import com.querydsl.core.types.dsl.BooleanExpression;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.transaction.Transactional;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class AlarmTargetService extends AlarmBaseService {

    @Autowired
    private AlarmTargetRepository alarmTargetRepository;
    @Autowired
    private AlarmItemService alarmItemService;

    @Autowired
    private CriteriaUtils criteriaUtils;

    private List<BooleanExpression> parse(ObjectNode criteria) {
        // TODO
        return null;
    }

    public PagedResponse<AlarmTargetModel> query(JacksonCriteria criteria) {
        // 默认查询未删除的数据
        criteria.add("valid", 1);
        PagedResponse<AlarmTarget> resp = criteriaUtils.query(QAlarmTarget.alarmTarget, this::parse, criteria);
        PagedResponse<AlarmTargetModel> result = new PagedResponse<>();
        result.setCount(resp.getCount());
        List<AlarmTarget> dataList = resp.getData();
        if (dataList != null && dataList.size() > 0) {
            Map<String, AlarmItemModel> itemCodeModelMap = getItemCodeModelMap(dataList);
            List<AlarmTargetModel> collect = dataList.stream().map(entity -> {
                AlarmTargetModel model = entity.toModel();
                // 获取报警条目的触发条件
                String itemCode = model.getItemCode();
                model.setCondition(getCondition(model, itemCodeModelMap.get(model.getItemCode())));
                return model;
            }).collect(Collectors.toList());
            result.setData(collect);
        }
        return result;
    }

    @Transactional
    public MapResponse create(AlarmTargetModel param) {
        MapResponse response = new MapResponse();

        AlarmParam alarmParam = prepareParam(response);
        if (alarmParam == null) {
            return response;
        }
        if (StringUtils.isEmpty(param.getItemCode())) {
            response.setFail("报警条目编码必填");
        }
        if (StringUtils.isEmpty(param.getClassCode())) {
            response.setFail("对象类型编码必填");
        }
        // 判断是否是重复创建
        JacksonCriteria criteria = JacksonCriteria.newInstance();
        criteria.add("itemCode", param.getItemCode());
        criteria.add("classCode", param.getClassCode());
        PagedResponse<AlarmTargetModel> result = query(criteria);
        if (result != null && result.getCount() > 0) {
            response.setFail("已有同类的匹配条件，请勿重复添加！");
            return response;
        }

        // 获取设备条目对象
        AlarmItemModel alarmItem = alarmItemService.getAlarmItemByItemCode(param.getItemCode());
        if (alarmItem == null) {
            response.setFail("无法获取到对应的报警条目，创建失败！");
            return response;
        }

        AlarmTarget entity = AlarmTarget.fromModel(param);
        entity.setId(UUID.randomUUID().toString());
        entity.setCreateUser(alarmParam.userId);
        entity.setCreateTime(new Date());
        entity.setProjectId(alarmParam.projectId);
        entity.setGroupCode(alarmParam.groupCode);
        // 继承alarmItem的一些属性
        if (StringUtils.isEmpty(entity.getLevel())) {
            entity.setLevel(alarmItem.getLevel());
        }
        alarmTargetRepository.save(entity);
        response.add("id", entity.getId());
        /* 拼装需要推送至消息队列的消息 */
        DmpMessage msg = new DmpMessage();
        msg.setMid(UUID.randomUUID().toString());
        msg.setType(EnumAlarmMessageType.ALARM_TARGET_CREATE.getValue());
        msg.setGroupCode(alarmParam.groupCode);
        msg.setProjectId(alarmParam.projectId);
        msg.setTargetId(entity.getId());
        msg.setStr1(entity.getItemCode());
        msg.setStr2(entity.getClassCode());
        msg.setAppId(alarmParam.appId);
        msg.setSendTime(DateUtils.format(new Date()));
        response.add(msg);
        return response;
    }

    public AlarmTarget get(String id) {
        Optional<AlarmTarget> result = alarmTargetRepository.findById(id);
        return result.orElse(null);
    }

    /**
     * @description: 判断匹配条件更新是否会影响报警定义
     * @param: param 匹配条件更新信息
     * @param: alarmTarget 匹配条件字段
     * @return: boolean
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/22 8:55 下午
     * @version: V1.0
     */
    private boolean influenceConfigs(AlarmTargetModel param, AlarmTarget alarmTarget) {
        boolean itemCodeChange = !StringUtils.isEmpty(param.getItemCode()) &&
                !param.getItemCode().equals(alarmTarget.getItemCode());
        boolean classCodeChange = !StringUtils.isEmpty(param.getClassCode()) &&
                !param.getClassCode().equals(alarmTarget.getClassCode());
        boolean concernChange = param.getConcern() != null &&
                !param.getConcern().equals(alarmTarget.getConcern());
        boolean openChange = param.getOpen() != null &&
                !param.getOpen().equals(alarmTarget.getOpen());
        boolean conditionChange = !StringUtils.isEmpty(param.getCondition()) &&
                !param.getCondition().equals(alarmTarget.getCondition());

        return itemCodeChange || classCodeChange || concernChange || openChange || conditionChange;
    }

    @Transactional
    public MapResponse update(AlarmTargetModel param) {
        MapResponse response = new MapResponse();
        AlarmTarget alarmTarget = get(param.getId());
        if (alarmTarget == null) {
            response.setFail("无法获取到要更新的数据！");
            return response;
        }
        AlarmParam alarmParam = prepareParam(response);
        if (alarmParam == null) {
            return response;
        }
        /* 判断是否会影响报警定义 */
        boolean influenceConfigs = influenceConfigs(param, alarmTarget);

        if (param.getItemCode() != null && !param.getItemCode().isEmpty()) {
            alarmTarget.setItemCode(param.getItemCode());
        }
        if (param.getClassCode() != null && !param.getClassCode().isEmpty()) {
            alarmTarget.setClassCode(param.getClassCode());
        }
        if (param.getConcern() != null) {
            alarmTarget.setConcern(param.getConcern());
        }
        if (param.getOpen() != null) {
            alarmTarget.setOpen(param.getOpen());
        }
        if (param.getCondition() != null && !param.getCondition().isEmpty()) {
            alarmTarget.setCondition(param.getCondition());
        }
        if (param.getValid() != null) {
            alarmTarget.setValid(param.getValid());
        }
        alarmTarget.setUpdateUser(alarmParam.userId);
        alarmTarget.setUpdateTime(new Date());
        alarmTargetRepository.save(alarmTarget);
        response.add("id", alarmTarget.getId());
        if (influenceConfigs) {
            /* 拼装需要推送至消息队列的消息 */
            DmpMessage msg = new DmpMessage();
            msg.setMid(UUID.randomUUID().toString());
            msg.setType(EnumAlarmMessageType.ALARM_TARGET_UPDATE.getValue());
            msg.setGroupCode(alarmParam.groupCode);
            msg.setProjectId(alarmParam.projectId);
            msg.setTargetId(alarmTarget.getId());
            msg.setStr1(alarmTarget.getItemCode());
            msg.setStr2(alarmTarget.getClassCode());
            msg.setAppId(alarmParam.appId);
            msg.setSendTime(DateUtils.format(new Date()));
            response.add(msg);
        }
        return response;
    }

    @Transactional
    public MapResponse delete(AlarmTargetModel param) {
        MapResponse response = new MapResponse();
        param.setValid(0);
        update(param);

        /* 拼装需要推送至消息队列的消息 */
        DmpMessage msg = new DmpMessage();
        msg.setMid(UUID.randomUUID().toString());
        msg.setType(EnumAlarmMessageType.ALARM_ITEM_DELETE.getValue());
        msg.setGroupCode(DmpParameterStorage.getGroupCode());
        msg.setProjectId(DmpParameterStorage.getProjectId());
        msg.setTargetId(param.getId());
        msg.setStr1(param.getItemCode());
        msg.setStr2(param.getClassCode());
        msg.setAppId(DmpParameterStorage.getAppId());
        msg.setSendTime(DateUtils.format(new Date()));
        response.add(msg);
        return response;
    }

    /**
     * @description: 根据给定的一组target，获取target所属的报警条目itemCode, alarmItem的映射map
     * @param: targetModelList
     * @return: java.util.HashMap<java.lang.String, com.persagy.dmp.alarm.model.AlarmItemModel>
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/20 2:32 下午
     * @version: V1.0
     */
    public Map<String, AlarmItemModel> getItemCodeModelMapByModelList(List<AlarmTargetModel> targetModelList) {
        List<String> itemCodeList = targetModelList.stream().map(AlarmTargetModel::getItemCode).collect(Collectors.toList());
        return alarmItemService.getItemCodeModelMap(itemCodeList);
    }

    /**
     * @description: 根据给定的一组target，获取target所属的报警条目itemCode, alarmItem的映射map
     * @param: targetList
     * @return: java.util.HashMap<java.lang.String, com.persagy.dmp.alarm.model.AlarmItemModel>
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/20 2:45 下午
     * @version: V1.0
     */
    public Map<String, AlarmItemModel> getItemCodeModelMap(List<AlarmTarget> targetList) {
        List<String> itemCodeList = targetList.stream().map(AlarmTarget::getItemCode).collect(Collectors.toList());
        return alarmItemService.getItemCodeModelMap(itemCodeList);
    }

    /**
     * @description: 获取匹配条件的触发条件：匹配条件自己的触发条件与所属报警条目的触发条件取并集
     * @param: alarmTargetModel 匹配条件对象
     * @param: alarmItemModel 报警条目对象
     * @return: ObjectNode
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/20 2:07 下午
     * @version: V1.0
     */
    public ObjectNode getCondition(AlarmTargetModel alarmTargetModel, AlarmItemModel alarmItemModel) {
        ObjectNode targetCondition = alarmTargetModel.getCondition();
        ObjectNode itemCondition = alarmItemModel.getCondition();
        return targetCondition == null ? itemCondition : targetCondition;
    }

    /**
     * @description: 根据报警条目编码和设备类型编码获取匹配条件对象
     * @param: itemCode 报警条目编码
     * @param: classCode 设备类型编码
     * @param: projectId 项目id
     * @return: com.persagy.dmp.alarm.model.AlarmTargetModel
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/21 11:53 上午
     * @version: V1.0
     */
    public AlarmTargetModel getAlarmTargetModel(String itemCode, String classCode, String projectId) {
        JacksonCriteria criteria = JacksonCriteria.newInstance();
        criteria.add("itemCode", itemCode);
        criteria.add("classCode", classCode);
        criteria.add("projectId", projectId);
        PagedResponse<AlarmTargetModel> pagedResponse = query(criteria);
        if (pagedResponse == null) {
            return null;
        }
        List<AlarmTargetModel> data = pagedResponse.getData();
        if (CollectionUtils.isEmpty(data)) {
            return null;
        }
        return data.get(0);
    }

    /**
     * @description: 根据报警条目编码获取匹配条件对象
     * @param: itemCode 报警条目编码
     * @param: projectId 项目id
     * @return: java.util.List<com.persagy.dmp.alarm.model.AlarmTargetModel>
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/21 12:13 下午
     * @version: V1.0
     */
    public List<AlarmTargetModel> getAlarmTargetModelsByItemCode(String itemCode, String projectId) {
        JacksonCriteria criteria = JacksonCriteria.newInstance();
        criteria.add("itemCode", itemCode);
        criteria.add("projectId", projectId);
        PagedResponse<AlarmTargetModel> pagedResponse = query(criteria);
        if (pagedResponse == null) {
            return null;
        }
        List<AlarmTargetModel> data = pagedResponse.getData();
        if (CollectionUtils.isEmpty(data)) {
            return null;
        }
        return data;
    }
}

