package com.persagy.dmp.rwd.datautils.service;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.JsonNodeFactory;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.json.JacksonMapper;
import com.persagy.dmp.common.http.HttpUtils;
import com.persagy.dmp.rwd.datautils.model.TransContext;
import com.persagy.dmp.rwd.enums.FuncidCategory;
import com.persagy.dmp.rwd.funcid.parse.parser.InfoCodeParser;
import com.persagy.dmp.rwd.funcid.parse.parser.InfoCodeParserFactory;
import com.persagy.dmp.rwd.funcid.parse.parser.ParserContext;
import com.persagy.dmp.rwd.model.FuncidDefModel;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.*;
import java.util.*;


/**
 * @author: yaoll
 * @date: 2020-10-14
 * @verison: 1.0
 */
@Slf4j
@Service
public class TransObjInfoService {

	@Value("${persagy.dmp.rwd.server}")
	private String rwdServer;

	@Autowired
	private HttpUtils httpUtils;

	@Autowired
	private InfoCodeParserFactory infoCodeParserFactory;

	@Data
	private static final class Model {
		private String project_id;
		private String obj_type;
		private String obj_id;
		private String info_id;
		private String s_value;
		private Long l_value;
		private Double d_value;
		private String time;
		private String error;

		@JsonIgnore
		public String getId() {
			if ("Pj".equals(obj_type)) {
				return obj_type + project_id;
			}
			return obj_type + project_id + obj_id;
		}
	}

	private static final Set<String> baseColumn = new HashSet<>(Arrays.asList(
			"BuildID",
			"FloorID",
			"RoomID",
			"EquipID",
			"BuildName",
			"FloorName",
			"RoomName",
			"EquipName",
			"BuildLocalID",
			"FloorLocalID",
			"RoomLocalID",
			"EquipLocalID",
			"BuildLocalName",
			"FloorLocalName",
			"RoomLocalName",
			"EquipLocalName"));

	public void trans(TransContext ctx) throws IOException {

		initClass(ctx);

		String file = ctx.getBasePath() + "/" + ctx.getDataFile();
		BufferedReader reader = new BufferedReader(new InputStreamReader(new FileInputStream(file)));

		String outFile = ctx.getBasePath() + "/" + ctx.getOutFile();
		PrintWriter outWriter = new PrintWriter(new OutputStreamWriter(new FileOutputStream(outFile)));

		String errorFile = ctx.getBasePath() + "/" + ctx.getErrorFile();
		PrintWriter errorWriter = new PrintWriter(new OutputStreamWriter(new FileOutputStream(errorFile)));

		Map<String, Map<String, FuncidDefModel>> classDefCache = new HashMap<>();

		Map<String, ObjectNode> objects = new HashMap<>();
		ArrayNode objectArray = JsonNodeFactory.instance.arrayNode();

		Map<String, String> objectClassCodeCache = new HashMap<>();
		initObjectClassCodeCache(ctx, objectClassCodeCache);


		int i = 0;
		String temp;
		while (true) {
			i++;
			temp = reader.readLine();
			if (temp == null) {
				break;
			}

			Model model = JacksonMapper.toObject(temp, Model.class);

			if ("Si".equalsIgnoreCase(model.obj_type)) {
				continue;
			}

			if ("Ec".equalsIgnoreCase(model.obj_type)) {
				continue;
			}

			String id = model.getId();
			if (!objectClassCodeCache.containsKey(id)) {
				model.error = "object not exists";
				log.error("obj error: {}", JacksonMapper.toSimpleJson(model));
				continue;
			}

			if (!objects.containsKey(id)) {
				ObjectNode obj = objectArray.addObject();
				obj.put("id", id);
				obj.put("projectId", ctx.getProjectId());
				obj.put("classCode", objectClassCodeCache.get(id));
				objects.put(id, obj);
			}

			ObjectNode obj = objects.get(id);
			parseBaseColumn(model, obj);

			String infoId = model.info_id;
			if (baseColumn.contains(infoId)) {
				outWriter.println(JacksonMapper.toSimpleJson(model));
				continue;
			}

			String classCode = JacksonMapper.getString(obj, "classCode");
			Map<String, FuncidDefModel> funcidCache = getFuncidCache(ctx, classDefCache, classCode);
			FuncidDefModel funcidDefModel = funcidCache.get(infoId);
			if (funcidDefModel == null) {
				model.error = "funcid def not exists";
				errorWriter.println(JacksonMapper.toSimpleJson(model));
				continue;
			}
			String code = funcidDefModel.getCode();
			Object modelValue = getValue(model);
			if (modelValue == null) {
				model.error = "value is null";
				errorWriter.println(JacksonMapper.toSimpleJson(model));
				continue;
			}
			ObjectNode temObj = getValueNode(model);
			if (funcidDefModel.getCategory() == FuncidCategory.STATIC) {
				InfoCodeParser parser = infoCodeParserFactory.getParser(funcidDefModel.getDataType());
				ParserContext parserContext = new ParserContext(funcidDefModel, temObj.get("value"));
				parser.checkValue(parserContext);
				if (parserContext.isHasError()) {
					model.error = parserContext.getError().toString();
					errorWriter.println(JacksonMapper.toSimpleJson(model));
				} else {
					outWriter.println(JacksonMapper.toSimpleJson(model));
					Object value = parserContext.getValue();
					obj.putPOJO(code, value);
				}
			} else {
				// 动态值不需校验
				outWriter.println(JacksonMapper.toSimpleJson(model));
				obj.set(code, temObj.get("value"));
			}

			if (i % 200 == 0) {
				outWriter.flush();
				errorWriter.flush();
			}
		}
		reader.close();

		outWriter.flush();
		outWriter.close();

		errorWriter.flush();
		errorWriter.close();

		if (objectArray.size() == 0) {
			return;
		}

		List<JsonNode> list = new LinkedList<>();
		for (JsonNode obj : objectArray) {
			list.add(obj);
			if (list.size() == 1000) {
				String url = rwdServer + "/rwd/instance/object/update?projectId=" + ctx.getProjectId() + "&groupCode=" + ctx.getGroupCode();
				HttpUtils.HttpResult response = httpUtils.post(url, JacksonMapper.toSimpleJson(list), 3600000);
				System.out.println(response.getContent());
				list = new LinkedList<>();
			}
		}
		if (list.size() > 0) {
			String url = rwdServer + "/rwd/instance/object/update?projectId=" + ctx.getProjectId() + "&groupCode=" + ctx.getGroupCode();
			HttpUtils.HttpResult response = httpUtils.post(url, JacksonMapper.toSimpleJson(list), 3600000);
			System.out.println(response.getContent());
		}
	}

	private void initObjectClassCodeCache(TransContext ctx, Map<String, String> objectClassCodeCache) {
		JacksonCriteria criteria = JacksonCriteria.newInstance();
		criteria.add("projectId", ctx.getProjectId());
		criteria.addWithColumn("classCode");
		String url = rwdServer + "/rwd/instance/object/query?projectId=" + ctx.getProjectId() + "&groupCode=" + ctx.getGroupCode();
		HttpUtils.HttpResult result = httpUtils.post(url, JacksonMapper.toSimpleJson(criteria), 1500000);
		if (result.success()) {
			String content = result.getContent();
			ObjectNode object = JacksonMapper.toObject(content, ObjectNode.class);
			ArrayNode data = JacksonMapper.getArray(object, "data", JsonNodeFactory.instance.arrayNode());

			for (JsonNode datum : data) {
				ObjectNode objectNode = (ObjectNode) datum;
				String id = JacksonMapper.getString(objectNode, "id");
				String classCode = JacksonMapper.getString(objectNode, "classCode");
				objectClassCodeCache.put(id, classCode);
			}
		}

	}

	private String getClassCode(Model model, TransContext ctx) {
		String obj_type = model.getObj_type();
		return ctx.getObjTypeMapping().get(obj_type);
	}

	private Map<String, FuncidDefModel> getFuncidCache(TransContext ctx, Map<String, Map<String, FuncidDefModel>> classDefCache, String classCode) {
		if (!classDefCache.containsKey(classCode)) {
			Map<String, FuncidDefModel> funcidCache = new HashMap<>();
			String url = rwdServer + "/rwd/def/funcid?projectId=" + ctx.getProjectId() + "&groupCode=" + ctx.getGroupCode();
			JacksonCriteria criteria = JacksonCriteria.newInstance();
			criteria.add("classCode", classCode);
			HttpUtils.HttpResult result = httpUtils.post(url, JacksonMapper.toSimpleJson(criteria), 300000);
			if (result.success()) {
				String content = result.getContent();
				ObjectNode object = JacksonMapper.toObject(content, ObjectNode.class);
				ArrayNode data = JacksonMapper.getArray(object, "data", JsonNodeFactory.instance.arrayNode());

				for (JsonNode datum : data) {
					ObjectNode funcJson = (ObjectNode) datum;
					FuncidDefModel funcidDefModel = JacksonMapper.toObject(funcJson.toString(), FuncidDefModel.class);
					funcidCache.put(funcidDefModel.getOrigCode(), funcidDefModel);
				}
			}
			classDefCache.put(classCode, funcidCache);
		}
		return classDefCache.get(classCode);
	}

	private void parseBaseColumn(Model model, ObjectNode obj) {
		String infoId = model.info_id;
		if ("BuildID".equals(infoId) || "FloorID".equals(infoId) || "RoomID".equals(infoId) || "EquipID".equals(infoId)) {
			return;
		} else if ("BuildName".equals(infoId) || "FloorName".equals(infoId) || "RoomName".equals(infoId) || "EquipName".equals(infoId)) {
			obj.put("name", model.s_value);
		} else if ("BuildLocalID".equals(infoId) || "FloorLocalID".equals(infoId) || "RoomLocalID".equals(infoId) || "EquipLocalID".equals(infoId)) {
			obj.put("localId", model.s_value);
		} else if ("BuildLocalName".equals(infoId) || "FloorLocalName".equals(infoId) || "RoomLocalName".equals(infoId) || "EquipLocalName".equals(infoId)) {
			obj.put("localName", model.s_value);
		}
	}

	private void initClass(TransContext ctx) {
		ArrayNode classes = queryClass(ctx);
		ctx.getObjTypeMapping().put("Bd", "building");
		ctx.getObjTypeMapping().put("Fl", "floor");
		ctx.getObjTypeMapping().put("Sp", "space");
		ctx.getObjTypeMapping().put("Eq", "equipment");

		ctx.getClassCodeMapping().put("Pj", "project");
		ctx.getClassCodeMapping().put("Bd", "building");
		ctx.getClassCodeMapping().put("Fl", "floor");
		ctx.getClassCodeMapping().put("Sy", "system");

		for (JsonNode cla : classes) {
			ObjectNode obj = (ObjectNode) cla;
			String parentCode = JacksonMapper.getString(obj, "parentCode", "");
			String code = JacksonMapper.getString(obj, "code");
			if ("equipment".equals(parentCode) || "system".equals(parentCode)) {
				ctx.getClassCodeMapping().put(code.substring(2), code);
			} else if ("space".equals(parentCode)) {
				ctx.getClassCodeMapping().put(code, code);
			}
		}
	}

	private ArrayNode queryClass(TransContext ctx) {
		String url = rwdServer + "/rwd/def/class?projectId=" + ctx.getProjectId() + "&groupCode=" + ctx.getGroupCode();
		JacksonCriteria criteria = JacksonCriteria.newInstance();
		HttpUtils.HttpResult result = httpUtils.post(url, JacksonMapper.toSimpleJson(criteria), 300000);
		if (result.success()) {
			String content = result.getContent();
			ObjectNode object = JacksonMapper.toObject(content, ObjectNode.class);
			ArrayNode data = JacksonMapper.getArray(object, "data");
			return data;
		} else {
			log.error("query class error: {}", result.getContent());
		}
		return JsonNodeFactory.instance.arrayNode();
	}

	private Object getValue(Model model) {
		if (model.s_value != null) {
			if ("".equals(model.s_value)) {
				return null;
			}
			return model.s_value;
		} else if (model.l_value != null) {
			return model.l_value;
		} else if (model.d_value != null) {
			return model.d_value;
		}
		return null;
	}

	private ObjectNode getValueNode(Model model) {
		ObjectNode obj = JsonNodeFactory.instance.objectNode();
		if (model.s_value != null) {
			if ("".equals(model.s_value)) {
				return obj;
			}
			return obj.put("value", model.s_value);
		} else if (model.l_value != null) {
			return obj.put("value", model.l_value);
		} else if (model.d_value != null) {
			return obj.put("value", model.d_value);
		}
		return obj;
	}

}
