package com.persagy.dmp.rwd.datautils.service;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.JsonNodeFactory;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.json.JacksonMapper;
import com.persagy.dmp.common.http.HttpUtils;
import com.persagy.dmp.rwd.datautils.model.TransContext;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.*;


/**
 * @author: yaoll
 * @date: 2020-10-14
 * @verison: 1.0
 */
@Slf4j
@Service
public class TransObjService {

	@Value("${persagy.dmp.rwd.server}")
	private String rwdServer;

	@Autowired
	private HttpUtils httpUtils;

	@Data
	public static class Model {
		private Boolean valid;
		private String obj_id;
		private String project_id;
		private String subtype;
		private String dtime;
		private String ctime;
		private String obj_type;

		@JsonIgnore
		public String getId() {
			if ("Pj".equals(obj_type)) {
				return obj_type + project_id;
			}
			return obj_type + project_id + obj_id;
		}

		public String getClassCode(TransContext ctx) {
			if ("Eq".equals(this.obj_type) || "Sy".equals(this.obj_type) || "Sp".equals(this.obj_type)) {
				return ctx.getClassCodeMapping().get(this.subtype);
			} else if ("Bd".equals(this.obj_type) || "Fl".equals(this.obj_type)) {
				return ctx.getClassCodeMapping().get(this.obj_type);
			}
			return null;
		}
	}

	private static final Set<String> baseColumn = new HashSet<>(Arrays.asList(
			"BuildID",
			"FloorID",
			"RoomID",
			"EquipID",
			"BuildName",
			"FloorName",
			"RoomName",
			"EquipName",
			"BuildLocalID",
			"FloorLocalID",
			"RoomLocalID",
			"EquipLocalID",
			"BuildLocalName",
			"FloorLocalName",
			"RoomLocalName",
			"EquipLocalName"));

	public void trans(TransContext ctx) throws IOException {

		initClass(ctx);

		Set<String> objectIds = initObjectIds(ctx);

		String file = ctx.getBasePath() + "/" + ctx.getDataFile();
		BufferedReader reader = new BufferedReader(new InputStreamReader(new FileInputStream(file)));

		ArrayNode objectArray = JsonNodeFactory.instance.arrayNode();
		int i = 0;
		String temp;
		while (true) {
			i++;
			temp = reader.readLine();
			if (temp == null) {
				break;
			}

			Model model = JacksonMapper.toObject(temp, Model.class);
			if ("Si".equalsIgnoreCase(model.obj_type)) {
				continue;
			}
			if ("Ec".equalsIgnoreCase(model.obj_type)) {
				continue;
			}

			String id = model.getId();
			if (objectIds.contains(id)) {
				continue;
			}
			String classCode = getClassCode(model, ctx);
			if (classCode == null) {
				log.info("error obj: {}", JacksonMapper.toSimpleJson(model));
				continue;
			}
			ObjectNode obj = objectArray.addObject();
			obj.put("id", id);
			obj.put("projectId", ctx.getProjectId());
			obj.put("objType", getObjType(model, ctx));
			obj.put("classCode", classCode);
			obj.put("createApp", "datautils");
		}
		reader.close();

		if (objectArray.size() > 0) {
			String url = rwdServer + "/rwd/instance/object/create?projectId=" + ctx.getProjectId() + "&groupCode=" + ctx.getGroupCode();
			HttpUtils.HttpResult response = httpUtils.post(url, objectArray.toString(), 3600000);
			System.out.println(response.getContent());
		}

	}

	private Set<String> initObjectIds(TransContext ctx) {
		Set<String> result = new HashSet<>();
		String url = rwdServer + "/rwd/instance/object/query?groupCode=" + ctx.getGroupCode() + "&projectId=" + ctx.getProjectId();
		JacksonCriteria criteria = JacksonCriteria.newInstance();
		criteria.add("groupCode", ctx.getGroupCode());
		criteria.add("projectId", ctx.getProjectId());
		criteria.addWithColumn("id");
		HttpUtils.HttpResult response = httpUtils.post(url, JacksonMapper.toSimpleJson(criteria), 3600000);
		if (response.success()) {
			String content = response.getContent();
			ObjectNode objectNode = JacksonMapper.toObject(content, ObjectNode.class);
			ArrayNode data = JacksonMapper.getArray(objectNode, "data", JsonNodeFactory.instance.arrayNode());
			for (JsonNode datum : data) {
				String id = JacksonMapper.getString((ObjectNode) datum, "id");
				result.add(id);
			}
		}
		return result;
	}

	private String getObjType(Model model, TransContext ctx) {
		String obj_type = model.getObj_type();
		return ctx.getObjTypeMapping().get(obj_type);
	}

	private String getClassCode(Model model, TransContext ctx) {
		return model.getClassCode(ctx);
	}

	private void initClass(TransContext ctx) {
		ArrayNode classes = queryClass(ctx);
//		for (JsonNode cla : classes) {
//			log.info(JacksonMapper.toSimpleJson(cla));
//		}
		ctx.getObjTypeMapping().put("Bd", "building");
		ctx.getObjTypeMapping().put("Fl", "floor");
		ctx.getObjTypeMapping().put("Sp", "space");
		ctx.getObjTypeMapping().put("Eq", "equipment");

		ctx.getClassCodeMapping().put("Pj", "project");
		ctx.getClassCodeMapping().put("Bd", "building");
		ctx.getClassCodeMapping().put("Fl", "floor");
		ctx.getClassCodeMapping().put("Sy", "system");

		for (JsonNode cla : classes) {
			ObjectNode obj = (ObjectNode) cla;
			String parentCode = JacksonMapper.getString(obj, "parentCode", "");
			String code = JacksonMapper.getString(obj, "code");
			if ("equipment".equals(parentCode) || "system".equals(parentCode)) {
				ctx.getClassCodeMapping().put(code.substring(2), code);
			} else if ("space".equals(parentCode)) {
				ctx.getClassCodeMapping().put(code, code);
			}
		}
	}

	private ArrayNode queryClass(TransContext ctx) {
		String url = rwdServer + "/rwd/def/class?projectId=" + ctx.getProjectId() + "&groupCode=" + ctx.getGroupCode();
		JacksonCriteria criteria = JacksonCriteria.newInstance();
		HttpUtils.HttpResult result = httpUtils.post(url, JacksonMapper.toSimpleJson(criteria), 30000);
		if (result.success()) {
			String content = result.getContent();
			ObjectNode object = JacksonMapper.toObject(content, ObjectNode.class);
			ArrayNode data = JacksonMapper.getArray(object, "data");
			return data;
		} else {
			log.error("query class error: {}", result.getContent());
		}
		return JsonNodeFactory.instance.arrayNode();
	}

}
