package com.persagy.dmp.rwd.funcid.parse.parser;

import com.fasterxml.jackson.databind.JavaType;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.JsonNodeType;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.common.json.JacksonMapper;
import com.persagy.dmp.rwd.enums.FuncidDataType;
import com.persagy.dmp.rwd.funcid.parse.model.RegionValue;
import com.persagy.dmp.rwd.funcid.parse.utils.ObjectNodeUtils;

import java.util.Arrays;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.List;
import java.util.stream.Collectors;

public class ParserUtils {

	/* ============== 类型是否匹配 ============= */

	// 是否是type对应类型的对象
	public static boolean is(JsonNode node, FuncidDataType type) {
		if (type == FuncidDataType.STRING) {
			return ObjectNodeUtils.isString(node);
		} else if (type == FuncidDataType.INTEGER) {
			return ObjectNodeUtils.isLong(node);
		} else if (type == FuncidDataType.DOUBLE) {
			return ObjectNodeUtils.isDouble(node);
		} else if (type == FuncidDataType.BOOLEAN) {
			return ObjectNodeUtils.isBoolean(node);
		} else if (type == FuncidDataType.DATETIME) { // TODO
			return ObjectNodeUtils.isString(node);
		} else if (type == FuncidDataType.ENUM) { // TODO
			return ObjectNodeUtils.isString(node);
		} else if (type == FuncidDataType.MENUM) { // TODO
			return ObjectNodeUtils.isString(node);
		} else if (type == FuncidDataType.ATTACHMENT) { // TODO
			return ObjectNodeUtils.isString(node);
		} else if (type == FuncidDataType.OBJECT) {
			return ObjectNodeUtils.isString(node); // TODO
		}
		return false;
	}

	// 是否是type对应类型的对象数组
	public static boolean isArray(JsonNode node, FuncidDataType type) {
		if (node.getNodeType() != JsonNodeType.ARRAY) {
			return false;
		}
		ArrayNode arr = (ArrayNode) node;
		for (JsonNode jsonNode : arr) {
			if (!is(jsonNode, type)) {
				return false;
			}
		}
		return true;
	}

	// 是否是type对应类型的对象区间
	public static boolean isRegion(JsonNode node, FuncidDataType type) {
		if (node.getNodeType() != JsonNodeType.OBJECT) {
			return false;
		}
		ObjectNode object = (ObjectNode) node;
		if (object.has("$gt") || object.has("$get") || object.has("$lt") || object.has("$lte")) {
			JsonNode child = object.get("$gt");
			if (child != null && !is(child, type)) {
				return false;
			}
			child = object.get("$gte");
			if (child != null && !is(child, type)) {
				return false;
			}
			child = object.get("$lt");
			if (child != null && !is(child, type)) {
				return false;
			}
			child = object.get("$gte");
			if (child != null && !is(child, type)) {
				return false;
			}
		} else {
			return false;
		}
		return true;
	}

	// 是否是type对应类型的对象区间数组
	public static boolean isRegionList(JsonNode node, FuncidDataType type) {
		if (node.getNodeType() != JsonNodeType.ARRAY) {
			return false;
		}
		ArrayNode arr = (ArrayNode) node;
		for (JsonNode jsonNode : arr) {
			if (!isRegion(jsonNode, type)) {
				return false;
			}
		}
		return true;
	}

	/* ============== 类型是否匹配 ============= */


	/**
	 * 解析约束区间
	 *
	 * @param dataSource
	 * @return
	 */
	public static <T> List<RegionValue<T>> parseDataSource(JsonNode dataSource, Class<T> type) {
//		System.out.println(dataSource == null ? "data source is null " : dataSource.toString());
		if (dataSource != null && dataSource.getNodeType() != JsonNodeType.NULL) {
			if (dataSource.getNodeType() == JsonNodeType.ARRAY) {
				JavaType javaType = JacksonMapper.nonEmptyMapper.getTypeFactory().constructParametricType(RegionValue.class, type);
				javaType = JacksonMapper.nonEmptyMapper.getTypeFactory().constructParametricType(List.class, javaType);
				List<RegionValue<T>> regionValues = JacksonMapper.toObject(dataSource.toString(), javaType);
				if (regionValues == null || regionValues.size() == 0) {
					return null;
				}
				List<RegionValue<T>> collect = regionValues.stream().filter(item -> !item.isEmpty()).collect(Collectors.toList());
				return collect.size() > 0 ? collect : null;

			} else if (dataSource.getNodeType() == JsonNodeType.OBJECT) {
				JavaType javaType = JacksonMapper.nonEmptyMapper.getTypeFactory().constructParametricType(RegionValue.class, type);
				RegionValue<T> region = JacksonMapper.toObject(dataSource.toString(), javaType);
				if (region == null || region.isEmpty()) {
					return null;
				} else {
					return Arrays.asList(region);
				}
			}
		}
		return null;
	}

	/**
	 * 尝试解析成区间list
	 *
	 * @param data
	 * @param type
	 * @param <T>
	 * @return
	 */
	public static <T> List<RegionValue<T>> parseToRegionList(String data, Class<T> type) {
		List<RegionValue<T>> list = null;
		if (data != null || !"".equals(data)) {
			JavaType javaType = JacksonMapper.nonEmptyMapper.getTypeFactory().constructParametricType(RegionValue.class, type);
			javaType = JacksonMapper.nonEmptyMapper.getTypeFactory().constructParametricType(List.class, javaType);
			list = JacksonMapper.toObject(data, javaType);
		}
		// 过滤掉空数据
		if (list == null || list.size() == 0) {
			return null;
		}
		List<RegionValue<T>> collect = list.stream().filter(item -> !item.isEmpty()).collect(Collectors.toList());
		return collect.size() > 0 ? collect : null;
	}

	/**
	 * 尝试解析成区间
	 *
	 * @param data
	 * @param type
	 * @param <T>
	 * @return
	 */
	public static <T> RegionValue<T> parseToRegion(String data, Class<T> type) {
		if (data != null || !"".equals(data)) {
			RegionValue<T> result = null;
			try {
				result = JacksonMapper.toObject(data.toString(), RegionValue.class, type);
			} catch (Exception e) {
				return null;
			}
			return result.isEmpty() ? null : result;
		}
		// 过滤掉空数据
		return null;
	}


	/**
	 * 尝试解析成list
	 *
	 * @param data
	 * @param type
	 * @param <T>
	 * @return
	 */
	public static <T> List<T> parseToList(String data, Class<T> type) {
		if (data != null || !"".equals(data)) {
			try {
				List<T> result = JacksonMapper.toObject(data.toString(), LinkedList.class, type);
				return result.isEmpty() ? null : result;
			} catch (Exception e) {
				return null;
			}
		}
		// 过滤掉空数据
		return null;
	}

	/**
	 * 尝试解析成对象
	 *
	 * @param data
	 * @param type
	 * @param <T>
	 * @return
	 */
	public static <T> T parse(String data, Class<T> type) {
		if (data != null || !"".equals(data)) {
			try {
				return JacksonMapper.toObject(data.toString(), type);
			} catch (Exception e) {
				return null;
			}
		}
		return null;
	}


	/**
	 * 区间匹配, source是否在target定义的区间内
	 *
	 * @param targetList
	 * @param sourceList
	 * @param <T>
	 * @return
	 */
	public static <T> boolean matchRegionList(List<RegionValue<T>> targetList, List<RegionValue<T>> sourceList, Comparator<T> comparator) {
		for (RegionValue<T> source : sourceList) {
			if (!matchRegion(targetList, source, comparator)) {
				return false;
			}
		}
		return true;
	}

	public static <T> boolean matchRegion(List<RegionValue<T>> targetList, RegionValue<T> source, Comparator<T> comparator) {
		for (RegionValue<T> target : targetList) {
			if (match(target, source, comparator)) {
				return true;
			}
		}
		return false;
	}

	// 目标集合的任一元素都要在任一区间内
	public static <T> boolean matchList(List<RegionValue<T>> targetList, List<T> sourceList, Comparator<T> comparator) {
		for (T t : sourceList) {
			if (!matchSingle(targetList, t, comparator)) {
				return false;
			}
		}
		return true;
	}

	// 目标值在任一区间内即可
	public static <T> boolean matchSingle(List<RegionValue<T>> targetList, T source, Comparator<T> comparator) {
		for (RegionValue<T> target : targetList) {
			if (match(target, source, comparator)) {
				return true;
			}
		}
		return false;
	}


	private static <T> boolean match(RegionValue<T> target, RegionValue<T> source, Comparator<T> comparator) {
		if (source.$gt != null && target.$gt != null && comparator.compare(source.$gt, target.$gt) < 0) {
			return false;
		}
		if (source.$gt != null && target.$gte != null && comparator.compare(source.$gt, target.$gte) < 0) {
			return false;
		}
		if (source.$gt != null && target.$lt != null && comparator.compare(source.$gt, target.$lt) >= 0) {
			return false;
		}
		if (source.$gt != null && target.$lte != null && comparator.compare(source.$gt, target.$lte) >= 0) {
			return false;
		}

		if (source.$gte != null && target.$gt != null && comparator.compare(source.$gte, target.$gt) <= 0) {
			return false;
		}
		if (source.$gte != null && target.$gte != null && comparator.compare(source.$gte, target.$gte) < 0) {
			return false;
		}
		if (source.$gte != null && target.$lt != null && comparator.compare(source.$gte, target.$lt) >= 0) {
			return false;
		}
		if (source.$gte != null && target.$lte != null && comparator.compare(source.$gte, target.$lte) > 0) {
			return false;
		}

		if (source.$lt != null && target.$gt != null && comparator.compare(source.$lt, target.$gt) <= 0) {
			return false;
		}
		if (source.$lt != null && target.$gte != null && comparator.compare(source.$lt, target.$gte) <= 0) {
			return false;
		}
		if (source.$lt != null && target.$lt != null && comparator.compare(source.$lt, target.$lt) > 0) {
			return false;
		}
		if (source.$lt != null && target.$lte != null && comparator.compare(source.$lt, target.$lte) > 0) {
			return false;
		}

		if (source.$lte != null && target.$gt != null && comparator.compare(source.$lte, target.$gt) <= 0) {
			return false;
		}
		if (source.$lte != null && target.$gte != null && comparator.compare(source.$lte, target.$gte) < 0) {
			return false;
		}
		if (source.$lte != null && target.$lt != null && comparator.compare(source.$lte, target.$lt) >= 0) {
			return false;
		}
		if (source.$lte != null && target.$lte != null && comparator.compare(source.$lte, target.$lte) > 0) {
			return false;
		}
		return true;
	}

	private static <T> boolean match(RegionValue<T> target, T source, Comparator<T> comparator) {
		if (target.$gt != null && comparator.compare(target.$gt, source) >= 0) {
			return false;
		}
		if (target.$gte != null && comparator.compare(target.$gte, source) > 0) {
			return false;
		}
		if (target.$lte != null && comparator.compare(target.$lte, source) < 0) {
			return false;
		}
		if (target.$lt != null && comparator.compare(target.$lt, source) <= 0) {
			return false;
		}
		return true;
	}

}
