package com.persagy.dmp.rwd.funcid.parse.parser.impl;

import com.fasterxml.jackson.databind.JsonNode;
import com.persagy.dmp.rwd.enums.FuncidDataType;
import com.persagy.dmp.rwd.funcid.parse.model.RegionValue;
import com.persagy.dmp.rwd.funcid.parse.parser.ParserContext;
import com.persagy.dmp.rwd.funcid.parse.parser.ParserUtils;
import com.persagy.dmp.rwd.model.FuncidDefModel;

import java.util.List;

/**
 * 日期时间解析器
 * <p>
 * 信息点的dataSource定义:<br/>
 * 单个区间: {"$gt": "0915", "$lt": "1031"} <br/>
 * 多个区间: [{"$gt": "0915", "$lt": "1031"}, {"$gt": "1201", "$lt": "1231"}] <br/>
 * 假设信息点编码的值为Info01, 入参格式:<br/>
 * 单数 非区间: { "Info01": "0915" }<br/>
 * 单数 区间:   { "Info01": { "$gt": "0915", "$lte": "1031"} }<br/>
 * 复数 非区间: { "Info01": [ "0915", "1031", "1201"] }<br/>
 * 复数 区间:  { "Info01": [{"$gt": "0915", "$lt": "1031"}, {"$gt": "1201", "$lt": "1231"}] }<br/>
 * 以上非区间情况下入参的任意一个值都必须在dataSource定义的区间中 <br/>
 * 区间情况下入参的任意一个区间都必须是dataSource定义的区间中的某个区间的子区间 <br/>
 * 日期时间格式由formatter定义，可以为“yyyyMMdd”等
 */
public class DateTimeParser extends AbstractParser<String> {

	@Override
	public void checkDef(ParserContext context) {
		super.checkDef(context);
	}

	@Override
	public void checkValue(ParserContext<String> context) {
		super.checkValue(context);
		if (context.isHasError()) {
			return;
		}

		FuncidDefModel def = context.getDef();
		JsonNode source = context.getSource();
		Boolean isMultiple = def.getIsMultiple();
		Boolean isRegion = def.getIsRegion();
		FuncidDataType dataType = def.getDataType();
		String sourceStr = source.toString();
		JsonNode dataSource = def.getDataSource();
		String dataSourceStr = dataSource == null ? "" : dataSource.toString();
		List<RegionValue<String>> regions = ParserUtils.parseDataSource(dataSource, String.class);
		String formater = def.getFormater();
		if (formater == null) {
			formater = "yyyyMMddHHmmss";
		}

		String error = "值错误, 需要[" + formater + "][" + dataSourceStr + "], 传入[" + sourceStr + "]";

		if (isMultiple && isRegion) {
			// 复数, 区间
			if (!ParserUtils.isRegionList(source, FuncidDataType.DATETIME)) {
				context.addError(error);
				return;
			}

			List<RegionValue<String>> value = ParserUtils.parseToRegionList(sourceStr, String.class);
			if (value == null || value.size() == 0 || !regionListMatchFormat(value, formater)) {
				context.addError(error);
				return;
			}
			// 区间匹配
			if (regions != null && !ParserUtils.matchRegionList(regions, value, String::compareTo)) {
				context.addError(error);
				return;
			}
			context.setRegionArray(value);
		}

		if (isMultiple && !isRegion) {
			// 复数 非区间
			if (!ParserUtils.isArray(source, dataType)) {
				context.addError(error);
				return;
			}

			List<String> value = ParserUtils.parseToList(sourceStr, String.class);
			if (value == null || value.size() == 0 || !listMatchFormat(value, formater)) {
				context.addError(error);
				return;
			}
			// 区间匹配
			if (regions != null && !ParserUtils.matchList(regions, value, String::compareTo)) {
				context.addError(error);
				return;
			}
			context.setArrayValue(value);
		}

		if (!isMultiple && isRegion) {
			// 单数 区间
			if (!ParserUtils.isRegion(source, dataType)) {
				context.addError(error);
				return;
			}
			RegionValue<String> value = ParserUtils.parseToRegion(sourceStr, String.class);
			if (value == null || value.isEmpty() || !regionMatchFormat(value, formater)) {
				context.addError(error);
				return;
			}
			// 区间匹配
			if (regions != null && !ParserUtils.matchRegion(regions, value, String::compareTo)) {
				context.addError(error);
				return;
			}
			context.setRegionValue(value);
		}

		if (!isMultiple && !isRegion) {
			// 单数 非区间
			if (!ParserUtils.is(source, dataType)) {
				context.addError(error);
				return;
			}
			String value = ParserUtils.parse(sourceStr, String.class);
			if (value == null || !matchFormat(value, formater)) {
				context.addError(error);
				return;
			}
			// 区间匹配
			if (regions != null && !ParserUtils.matchSingle(regions, value, String::compareTo)) {
				context.addError(error);
				return;
			}
			context.setValue(value);
		}
	}

	private boolean matchFormat(String value, String formater) {
		if (formater == null) {
			return true;
		}
		return formater.length() == value.length();
	}

	private boolean regionMatchFormat(RegionValue<String> region, String formater) {
		if (formater == null) {
			return true;
		}
		if (region.$gt != null && !matchFormat(region.$gt, formater)) {
			return false;
		}
		if (region.$gte != null && !matchFormat(region.$gte, formater)) {
			return false;
		}
		if (region.$lte != null && !matchFormat(region.$lte, formater)) {
			return false;
		}
		if (region.$lt != null && !matchFormat(region.$lt, formater)) {
			return false;
		}
		return true;
	}

	private boolean listMatchFormat(List<String> values, String formater) {
		if (formater == null) {
			return true;
		}
		for (String value : values) {
			if (!matchFormat(value, formater)) {
				return false;
			}
		}
		return true;
	}

	private boolean regionListMatchFormat(List<RegionValue<String>> regions, String formater) {
		if (formater == null) {
			return true;
		}
		for (RegionValue<String> region : regions) {
			if (!regionMatchFormat(region, formater)) {
				return false;
			}
		}
		return true;
	}
}

