package com.persagy.dmp.rwd.funcid.parse.parser.impl;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.JsonNodeType;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.dmp.rwd.enums.FuncidDataType;
import com.persagy.dmp.rwd.funcid.parse.parser.ParserContext;
import com.persagy.dmp.rwd.funcid.parse.parser.ParserUtils;
import com.persagy.dmp.rwd.model.FuncidDefModel;
import lombok.Getter;
import lombok.Setter;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

/**
 * @author: yaoll
 * @date: 2020-09-11
 * @verison: 1.0
 */
public class MEnumParser extends AbstractParser<List<String>> {

	@Override
	public void checkDef(ParserContext context) {
		super.checkDef(context);
	}

	@Override
	public void checkValue(ParserContext<List<String>> context) {
		super.checkValue(context);
		if (context.isHasError()) {
			return;
		}

		FuncidDefModel def = context.getDef();
		JsonNode source = context.getSource();
		FuncidDataType dataType = def.getDataType();
		String sourceStr = source.toString();
		JsonNode dataSource = def.getDataSource();
		String dataSourceStr = dataSource == null ? "" : dataSource.toString();
		Map<String, EnumNode> enumValues = getEnumValues(dataSource);
		if (enumValues == null) {
			context.addError("dataSource[" + dataSourceStr + "]错误");
			return;
		}

		String error = "值错误, 需要[" + dataSourceStr + "], 传入[" + sourceStr + "]";

		Boolean isMultiple = def.getIsMultiple();
		Boolean isRegion = def.getIsRegion();
		if (isMultiple && isRegion) {
			// 复数, 区间
			context.addError("不支持[Enum 复数 区间]");
			return;
		}

		if (isMultiple && !isRegion) {
			// 复数 非区间 二维数组
			if (!isMatrix(source, dataType)) {
				context.addError(error);
				return;
			}

			List<List<String>> value = toMatrix(dataSource, FuncidDataType.MENUM);
			if (value == null || value.size() == 0) {
				context.addError(error);
				return;
			}
			for (List<String> subValues : value) {
				for (String code : subValues) {
					if (!enumValues.containsKey(code)) {
						context.addError(error);
						return;
					}
				}
			}
			context.setArrayValue(value);
		}

		if (!isMultiple && isRegion) {
			// 单数, 区间
			context.addError("不支持[Enum 单数 区间]");
			return;
		}

		if (!isMultiple && !isRegion) {
			// 单数 非区间
			if (!ParserUtils.isArray(source, dataType)) {
				context.addError(error);
				return;
			}

			List<String> value = ParserUtils.parseToList(sourceStr, String.class);
			if (value == null || value.size() == 0) {
				context.addError(error);
				return;
			}
			for (String code : value) {
				if (!enumValues.containsKey(code)) {
					context.addError(error);
					return;
				}
			}
			context.setValue(value);
		}
	}

	@Getter
	@Setter
	public class EnumNode {
		private String code;
		private String parentCode;

		public EnumNode(String code, String parentCode) {
			this.code = code;
			this.parentCode = parentCode;
		}
	}

	private Map<String, EnumNode> getEnumValues(JsonNode dataSource) {
		if (dataSource == null) {
			return null;
		}
		JsonNodeType nodeType = dataSource.getNodeType();
		if (nodeType != JsonNodeType.ARRAY) {
			return null;
		}
		Map<String, EnumNode> result = new HashMap<>();
		ArrayNode array = (ArrayNode) dataSource;
		for (JsonNode jsonNode : array) {
			ObjectNode node = (ObjectNode) jsonNode;
			String code = node.has("code") ? node.get("code").asText() : null;
			String parentCode = node.has("parentCode") ? node.get("parentCode").asText() : null;
			if (code != null) {
				result.put(code, new EnumNode(code, parentCode));
			}
		}
		return result;
	}

	/**
	 * 是否是type对应类型的对象二维数组
	 *
	 * @param node
	 * @param type
	 * @return
	 */
	private boolean isMatrix(JsonNode node, FuncidDataType type) {
		if (node.getNodeType() != JsonNodeType.ARRAY) {
			return false;
		}
		ArrayNode arr = (ArrayNode) node;
		for (JsonNode jsonNode : arr) {
			if (jsonNode.getNodeType() != JsonNodeType.ARRAY) {
				return false;
			}
			ArrayNode sub = (ArrayNode) jsonNode;
			for (JsonNode innerNode : sub) {
				if (!ParserUtils.is(innerNode, type)) {
					return false;
				}
			}
		}
		return true;
	}

	private List<List<String>> toMatrix(JsonNode node, FuncidDataType type) {
		List<List<String>> value = new LinkedList<>();
		ArrayNode arr = (ArrayNode) node;
		for (JsonNode jsonNode : arr) {
			ArrayNode sub = (ArrayNode) jsonNode;
			List<String> subValue = new LinkedList<>();
			for (JsonNode innerNode : sub) {
				subValue.add(innerNode.asText());
			}
			value.add(subValue);
		}
		return value;
	}
}
