package com.persagy.dmp.rwd.funcid.parse.parser.impl;

import com.fasterxml.jackson.databind.JsonNode;
import com.persagy.dmp.rwd.enums.FuncidDataType;
import com.persagy.dmp.rwd.funcid.parse.model.RegionValue;
import com.persagy.dmp.rwd.funcid.parse.parser.ParserContext;
import com.persagy.dmp.rwd.funcid.parse.parser.ParserUtils;
import com.persagy.dmp.rwd.model.FuncidDefModel;

import java.util.Comparator;
import java.util.List;

/**
 * 信息点校验
 * <p>
 * <p>
 * 整型解析器
 * <p>
 * 信息点的dataSource定义:<br/>
 * 单个区间: {"$gt": 10, "$lt": 50} <br/>
 * 多个区间: [{"$gt": 10, "$lt": 50}, {"$gt": 210, "$lt": 250}] <br/>
 * 假设信息点编码的值为Info01, 入参格式:<br/>
 * 单数 非区间: { "Info01": 16 }<br/>
 * 单数 区间:   { "Info01": { "$gt": 15, "$lte": 21} }<br/>
 * 复数 非区间: { "Info01": [ 16, 19, 21] }<br/>
 * 复数 区间:  { "Info01": [{ "$gt": 15, "$lte": 21}, { "$gt": 24, "$lte": 29}, { "$gt": 40, "$lte": 44}] }<br/>
 * 以上非区间情况下入参的任意一个值都必须在dataSource定义的区间中 <br/>
 * 区间情况下入参的任意一个区间都必须是dataSource定义的区间中的某个区间的子区间
 */
public class NumberParser<T> extends AbstractParser<T> {

	protected Class<T> getClassType() {
		return null;
	}

	protected Comparator<T> getComparator() {
		return null;
	}

	/**
	 * 校验信息点定义
	 *
	 * @param context
	 */
	@Override
	public void checkDef(ParserContext context) {
	}

	/**
	 * 检验信息点的值
	 *
	 * @param context
	 */
	@Override
	public void checkValue(ParserContext<T> context) {
		super.checkValue(context);
		if (context.isHasError()) {
			return;
		}

		FuncidDefModel def = context.getDef();
		JsonNode source = context.getSource();
		Boolean isMultiple = def.getIsMultiple();
		Boolean isRegion = def.getIsRegion();
		FuncidDataType dataType = def.getDataType();
		String sourceStr = source.toString();
		JsonNode dataSource = def.getDataSource();
		String dataSourceStr = dataSource == null ? "" : dataSource.toString();
		List<RegionValue<T>> regions = ParserUtils.parseDataSource(dataSource, getClassType());

		String error = "值错误, 需要[" + dataSourceStr + "], 传入[" + sourceStr + "]";
		if (isMultiple && isRegion) {
			// 复数, 区间
			if (!ParserUtils.isRegionList(source, dataType)) {
				context.addError(error);
				return;
			}

			List<RegionValue<T>> value = ParserUtils.parseToRegionList(sourceStr, getClassType());
			if (value == null || value.size() == 0) {
				context.addError(error);
				return;
			}
			// 区间匹配
			if (regions != null && !ParserUtils.matchRegionList(regions, value, getComparator())) {
				context.addError(error);
				return;
			}
			context.setRegionArray(value);
		}

		if (isMultiple && !isRegion) {
			// 复数 非区间
			if (!ParserUtils.isArray(source, dataType)) {
				context.addError(error);
				return;
			}

			List<T> value = ParserUtils.parseToList(sourceStr, getClassType());
			if (value == null || value.size() == 0) {
				context.addError(error);
				return;
			}
			// 区间匹配
			if (regions != null && !ParserUtils.matchList(regions, value, getComparator())) {
				context.addError(error);
				return;
			}
			context.setArrayValue(value);
		}

		if (!isMultiple && isRegion) {
			// 单数 区间
			if (!ParserUtils.isRegion(source, dataType)) {
				context.addError(error);
				return;
			}
			RegionValue<T> value = ParserUtils.parseToRegion(sourceStr, getClassType());
			if (value == null || value.isEmpty()) {
				context.addError(error);
				return;
			}
			// 区间匹配
			if (regions != null && !ParserUtils.matchRegion(regions, value, getComparator())) {
				context.addError(error);
				return;
			}
			context.setRegionValue(value);
		}

		if (!isMultiple && !isRegion) {
			// 单数 非区间
			if (!ParserUtils.is(source, dataType)) {
				context.addError(error);
				return;
			}
			T value = ParserUtils.parse(sourceStr, getClassType());
			if (value == null) {
				context.addError(error);
				return;
			}
			// 区间匹配
			if (regions != null && !ParserUtils.matchSingle(regions, value, getComparator())) {
				context.addError(error);
				return;
			}
			context.setValue(value);
		}
	}

}
