package com.persagy.dmp.rwd.funcid.parse.parser.impl;

import com.fasterxml.jackson.databind.JsonNode;
import com.persagy.dmp.rwd.enums.FuncidDataType;
import com.persagy.dmp.rwd.funcid.parse.model.RegionValue;
import com.persagy.dmp.rwd.funcid.parse.parser.ParserContext;
import com.persagy.dmp.rwd.funcid.parse.parser.ParserUtils;
import com.persagy.dmp.rwd.model.FuncidDefModel;

import java.util.List;
import java.util.stream.Collectors;

/**
 * 字符型解析器
 * <p>
 * 信息点的dataSource定义:<br/>
 * 区间定义为字符串长度
 * 单个区间: {"$gt": 10, "$lt": 50} <br/>
 * 多个区间: [{"$gt": 10, "$lt": 50}, {"$gt": 210, "$lt": 250}] <br/>
 * 假设信息点编码的值为Info01, 入参格式:<br/>
 * 单数 非区间: { "Info01": “abc” }<br/>
 * 复数 非区间: { "Info01": [ "a", “b”, "c"] }<br/>
 * 以上非区间情况下入参的任意一个值的长度都必须在dataSource定义的区间中 <br/>
 */
public class StringParser extends AbstractParser<String> {

	@Override
	public void checkDef(ParserContext context) {
		super.checkDef(context);
	}

	@Override
	public void checkValue(ParserContext<String> context) {
		super.checkValue(context);
		if (context.isHasError()) {
			return;
		}

		FuncidDefModel def = context.getDef();
		JsonNode source = context.getSource();

		FuncidDataType dataType = def.getDataType();
		String sourceStr = source.toString();
		JsonNode dataSource = def.getDataSource();
		String dataSourceStr = dataSource == null ? "" : dataSource.toString();
		List<RegionValue<Integer>> regions = ParserUtils.parseDataSource(dataSource, Integer.class);
		String formater = def.getFormater();
		if (formater == null) {
			formater = "yyyyMMddHHmmss";
		}

		String error = "值错误, 需要[" + formater + "][" + dataSourceStr + "], 传入[" + sourceStr + "]";

		Boolean isMultiple = def.getIsMultiple();
		Boolean isRegion = def.getIsRegion();
		if (isMultiple && isRegion) {
			// 复数, 区间
			context.addError("不支持[String 复数 区间]");
			return;
		}

		if (isMultiple && !isRegion) {
			// 复数 非区间
			if (!ParserUtils.isArray(source, dataType)) {
				context.addError(error);
				return;
			}

			List<String> value = ParserUtils.parseToList(sourceStr, String.class);
			if (value == null || value.size() == 0) {
				context.addError(error);
				return;
			}
			// 区间匹配
			List<Integer> collect = value.stream().map(String::length).collect(Collectors.toList());
			if (regions != null && !ParserUtils.matchList(regions, collect, Integer::compareTo)) {
				context.addError(error);
				return;
			}
			context.setArrayValue(value);
		}

		if (!isMultiple && isRegion) {
			// 单数 区间
			context.addError("不支持[String 单数 区间]");
			return;
		}

		if (!isMultiple && !isRegion) {
			// 单数 非区间
			if (!ParserUtils.is(source, dataType)) {
				context.addError(error);
				return;
			}
			String value = ParserUtils.parse(sourceStr, String.class);
			// 区间匹配
			if (regions != null && !ParserUtils.matchSingle(regions, value.length(), Integer::compareTo)) {
				context.addError(error);
				return;
			}
			context.setValue(value);
		}
	}


}
