package com.persagy.dmp.rwd.config;

import com.persagy.common.json.JacksonMapper;
import com.persagy.common.web.BaseResponse;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.*;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * @author: yaoll
 * @date: 2020-09-22
 * @verison: 1.0
 */
@Slf4j
@Aspect
@Component
public class RequestLogAspect {

	private static final ThreadLocal<LogData> dataStorage = new ThreadLocal();

	private static final AtomicInteger flag = new AtomicInteger(10000000);


	@Pointcut("execution(public * com.persagy.dmp.*.controller.*.*(..))")
	public void controllerPoint() {
	}

	@Before("controllerPoint()")
	public void doBefore(JoinPoint joinPoint) {
		ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
		HttpServletRequest request = attributes.getRequest();
		LogData data = initFlag(request);
		dataStorage.set(data);
		log.info("{} REQUEST URL      : {} {}", data.intFlag, data.method, data.url);
		log.info("{} REQUEST ARGS     : {}", data.intFlag, JacksonMapper.toSimpleJson(prepare(joinPoint)));
	}

	public static List<Object> prepare(JoinPoint joinPoint) {
		Object[] args = joinPoint.getArgs();
		List<Object> list = new LinkedList<>();
		if (args != null && args.length > 0) {
			for (Object arg : args) {
				if (arg instanceof ServletRequest) {
					continue;
				}
				if (arg instanceof ServletResponse) {
					continue;
				}
				if (arg instanceof MultipartFile) {
					continue;
				}
				list.add(arg);
			}
		}
		return list;
	}

	@AfterReturning(returning = "ret", pointcut = "controllerPoint()")
	public void doAfterReturning(Object ret) {

		LogData data = dataStorage.get();
		if (ret != null) {
			if (ret instanceof BaseResponse) {
				log.info("{} REQUEST DURATION : {} {} {}", data.intFlag, System.currentTimeMillis() - data.timestamp, ((BaseResponse) ret).getResult(), data.url);
			} else {
				log.info("{} REQUEST DURATION : {} {} {}", data.intFlag, System.currentTimeMillis() - data.timestamp, ret.getClass().getName(), data.url);
			}
		} else {
			log.info("{} REQUEST DURATION : {} {} {}", data.intFlag, System.currentTimeMillis() - data.timestamp, "null", data.url);
		}
		dataStorage.remove();
	}

	@AfterThrowing(throwing = "ex", pointcut = "controllerPoint()")
	public void doAfterThrowing(Throwable ex) {
		LogData data = dataStorage.get();
		log.error("{} REQUEST EXCEPTION: {} {} {}", data.intFlag, System.currentTimeMillis() - data.timestamp, data.url, ex.getMessage());
		dataStorage.remove();
	}

	private static LogData initFlag(HttpServletRequest request) {
		LogData data = new LogData();
		data.timestamp = System.currentTimeMillis();
		data.method = request.getMethod();

		String requestURI = request.getRequestURI();
		String queryString = request.getQueryString();
		data.url = requestURI + (queryString == null ? "" : "?" + queryString);

		int result = flag.addAndGet(1);
		if (result >= 99999999) {
			flag.set(10000000);
		}
		data.intFlag = result;
		dataStorage.set(data);
		return data;
	}

	private static class LogData {
		Integer intFlag;
		Long timestamp;
		String url;
		String method;
	}
}
