package com.persagy.dmp.rwd.service;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.json.JacksonMapper;
import com.persagy.common.web.ListResponse;
import com.persagy.common.web.MapResponse;
import com.persagy.common.web.PagedResponse;
import com.persagy.dmp.rwd.entity.*;
import com.persagy.dmp.rwd.enums.FuncidCategory;
import com.persagy.dmp.rwd.enums.FuncidDataType;
import com.persagy.dmp.rwd.model.ClassDefModel;
import com.persagy.dmp.rwd.model.FuncidDefModel;
import com.persagy.dmp.rwd.util.ClassCodeUtils;
import com.querydsl.core.types.dsl.BooleanExpression;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service
public class FuncidDefService extends BaseService {

	@Autowired
	private ClassDefService classDefService;

	@Autowired
	private ClassCodeUtils classCodeUtils;

	private List<BooleanExpression> parse(ObjectNode object) {
		// TODO
		List<BooleanExpression> list = new LinkedList<>();
		QFuncidDef qt = QFuncidDef.funcidDef;
		list.addAll(criteriaUtils.parse(qt.category, object.get("category"), FuncidCategory.class));
		list.addAll(criteriaUtils.parse(qt.dataType, object.get("dataType"), FuncidDataType.class));
		return list;
	}

	public PagedResponse<FuncidDefModel> query(JacksonCriteria criteria) {
		// 预处理criteria
		classCodeUtils.prepareCriteriaForFuncid(criteria, "classCode");

		PagedResponse<FuncidDef> resp = criteriaUtils.query(QFuncidDef.funcidDef, this::parse, criteria);
		PagedResponse<FuncidDefModel> result = new PagedResponse<>();
		result.setCount(resp.getCount());
		List<FuncidDef> dataList = resp.getData();
		if (dataList != null && dataList.size() > 0) {
			List<FuncidDefModel> collect = dataList.stream().map(entity -> {
				FuncidDefModel model = entity.toModel();
				return model;
			}).collect(Collectors.toList());
			result.setData(collect);
		}
		return result;
	}


	public ListResponse<FuncidDefModel> queryFuncid(String groupCode, String schemaId, String projectId, String appId, JacksonCriteria request) {
		ListResponse<FuncidDefModel> resp = new ListResponse<>();
		log.info("queryFuncid: {} {} {} {} {}", groupCode, schemaId, projectId, appId, JacksonMapper.toSimpleJson(request));
		OrgParam orgParam = super.prepareParam(resp);
		if (!resp.success()) {
			return resp;
		}

		if (orgParam.appId != null && orgParam.projectId == null) {
			resp.setFail("传入appId时必须同时传入projectId");
			return resp;
		}

		if (orgParam.projectId != null) {
			if (orgParam.groupCode == null || "0".equals(orgParam.groupCode)) {
				resp.setFail("传入porjectId时必须同时传入groupCode");
				return resp;
			}
			QSchemaProject pqt = QSchemaProject.schemaProject;
			Optional<SchemaProject> one = schemaProjectRepository.findOne(pqt.projectId.eq(orgParam.projectId).and(pqt.groupCode.eq(orgParam.groupCode)));
			if (one.isPresent()) {
				orgParam.schemaId = one.get().getSchemaId();
			} else {
				resp.setFail("当前项目未配置集团方案");
				return resp;
			}
		}

		List<FuncidDefModel> list = null;
		boolean hasClassCode = request.getCriteria().has("classCode");
		if (hasClassCode) {
			list = queryWithClassCode(orgParam, resp, request);
		} else {
			list = queryWithoutClassCode(orgParam, request);
		}

		// 过滤方案订阅
		if (orgParam.schemaId != null && !"0".equalsIgnoreCase(orgParam.schemaId)) {
			Map<String, Set<String>> classInfoCodeCache = new HashMap<>();
			Iterator<FuncidDefModel> iterator = list.iterator();
			while (iterator.hasNext()) {
				FuncidDefModel funcid = iterator.next();
				String classCode = funcid.getClassCode();
				String infoCode = funcid.getCode();
				if (!classInfoCodeCache.containsKey(classCode)) {
					QSchemaFuncid qsf = QSchemaFuncid.schemaFuncid;
					Iterable<SchemaFuncid> all = schemaFuncidRepository.findAll(qsf.groupCode.eq(orgParam.groupCode).and(qsf.schemaId.eq(orgParam.schemaId).and(qsf.classCode.eq(classCode))));
					Set<String> set = new HashSet<>();
					for (SchemaFuncid schemaFuncid : all) {
						set.add(schemaFuncid.getInfoCode());
					}
					classInfoCodeCache.put(classCode, set);
				}
				Set<String> infoCodes = classInfoCodeCache.get(classCode);
				if (infoCodes.size() > 0 && !infoCodes.contains(infoCode)) {
					iterator.remove();
				}
			}
		}

		// 过滤应用订阅
		if (orgParam.appId != null && !"0".equalsIgnoreCase(orgParam.appId)) {
			Map<String, Set<String>> classInfoCodeCache = new HashMap<>();
			Iterator<FuncidDefModel> iterator = list.iterator();
			while (iterator.hasNext()) {
				FuncidDefModel funcid = iterator.next();
				String classCode = funcid.getClassCode();
				String infoCode = funcid.getCode();
				if (!classInfoCodeCache.containsKey(classCode)) {
					QAppFuncid qaf = QAppFuncid.appFuncid;
					Iterable<AppFuncid> all = appFuncidRepository.findAll(qaf.appId.eq(orgParam.appId)
							.and(qaf.groupCode.eq(orgParam.groupCode))
							.and(qaf.projectId.eq(orgParam.projectId))
							.and(qaf.classCode.eq(classCode)));
					Set<String> set = new HashSet<>();
					for (AppFuncid AppFuncid : all) {
						set.add(AppFuncid.getInfoCode());
					}
					classInfoCodeCache.put(classCode, set);
				}
				Set<String> infoCodes = classInfoCodeCache.get(classCode);
				if (infoCodes.size() > 0 && !infoCodes.contains(infoCode)) {
					iterator.remove();
				}
			}
		}

		resp.setData(list);
		return resp;
	}

	private List<FuncidDefModel> queryWithClassCode(OrgParam orgParam, ListResponse<FuncidDefModel> resp, JacksonCriteria request) {
		Map<String, FuncidDefModel> cache = new HashMap<>();
		JsonNode classCode = request.getCriteria().get("classCode");

		JacksonCriteria criteria = JacksonCriteria.newInstance();
		criteria.getCriteria().set("code", classCode);
		criteria.setWithoutCount(true);
		System.out.println("query class " + JacksonMapper.toSimpleJson(criteria));
		ListResponse<ClassDefModel> classDefModelListResponse = classDefService.queryClass(orgParam.groupCode, orgParam.schemaId, orgParam.projectId, orgParam.appId, false, criteria);
		List<ClassDefModel> classDefModelList = classDefModelListResponse.getData();
		List<FuncidDefModel> list = new LinkedList<>();
		if (classDefModelList.size() > 0) {
			for (ClassDefModel classDefModel : classDefModelList) {
				JacksonCriteria funcidCriteria = JacksonCriteria.newInstance();
				funcidCriteria.getCriteria().setAll(request.getCriteria());
				if (classDefModel.getParentCode() != null) {
					funcidCriteria.add("classCode").in(Arrays.asList(classDefModel.getCode(), classDefModel.getParentCode()));
				} else {
					funcidCriteria.add("classCode", classDefModel.getCode());
				}
				List<FuncidDefModel> funcidDefModels = queryWithoutClassCode(orgParam, funcidCriteria);
				funcidDefModels.forEach(model -> model.setClassCode(classDefModel.getCode()));
				list.addAll(funcidDefModels);
			}
		}
		return list;
	}

	private List<FuncidDefModel> queryWithoutClassCode(OrgParam orgParam, JacksonCriteria request) {
		Map<String, FuncidDefModel> cache = new HashMap<>();
		// 预处理criteria
		classCodeUtils.prepareCriteriaForFuncid(request, "classCode");

		ObjectNode requestCriteria = request.getCriteria();
		// 查询平台级
		JacksonCriteria criteria = JacksonCriteria.newInstance();
		criteria.getCriteria().setAll(requestCriteria);
		criteria.add("type", "common");
		criteria.add("groupCode", "0");
		criteria.setWithoutCount(true);
		PagedResponse<FuncidDefModel> queryResp = query(criteria);
		queryResp.getData().forEach(model -> cache.put(model.getClassCode() + "-" + model.getCode(), model));

		// 查询集团级
		if (orgParam.groupCode != null && !"0".equalsIgnoreCase(orgParam.groupCode)) {
			criteria = JacksonCriteria.newInstance();
			criteria.getCriteria().setAll(requestCriteria);
			criteria.add("type", "group");
			criteria.add("groupCode", orgParam.groupCode);
			criteria.setWithoutCount(true);
			queryResp = query(criteria);
			queryResp.getData().forEach(model -> cache.put(model.getClassCode() + "-" + model.getCode(), model));
		}
		// 查询项目级
		if (orgParam.projectId != null && !"0".equalsIgnoreCase(orgParam.projectId)) {
			criteria = JacksonCriteria.newInstance();
			criteria.getCriteria().setAll(requestCriteria);
			criteria.add("type", "project");
			criteria.add("groupCode", orgParam.groupCode);
			criteria.add("projectId", orgParam.projectId);
			criteria.setWithoutCount(true);
			queryResp = query(criteria);
			queryResp.getData().forEach(model -> cache.put(model.getClassCode() + "-" + model.getCode(), model));
		}

		// 继承父级信息点
		List<FuncidDefModel> list = new LinkedList<>();
		list.addAll(cache.values());
		return list;
	}

	@Transactional
	public MapResponse create(FuncidDefModel param) {
		MapResponse response = new MapResponse();
		// TODO
		FuncidDef entity = FuncidDef.fromModel(param);
		funcidDefRepository.save(entity);
		response.add("id", entity.getId());
		return response;
	}

	@Transactional
	public MapResponse update(FuncidDefModel param) {
		MapResponse response = new MapResponse();
		// TODO
		FuncidDef entity = FuncidDef.fromModel(param);
		funcidDefRepository.save(entity);
		response.add("id", entity.getId());
		return response;
	}

	@Transactional
	public MapResponse delete(FuncidDefModel param) {
		MapResponse response = new MapResponse();
		// TODO
		funcidDefRepository.deleteById(param.getId());
		return response;
	}


}

